#! /bin/bash

# This file is part of cardwords
# (c) 1999 Tobias Peters
# see file COPYING for the copyright terms.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# // cardwords_sgml2helptext.sh



# Script to extract the help texts from the README.
# Uses sed-script cardwords_sgml2helptext.sed
# and c++-prog cardwords_sgml2helptext.cc .

sed -n -e "/<\!-- BEGIN_HELP_TEXT $1 -->/,/<\!-- END_HELP_TEXT $1 -->/ p" < README.sgml \
| sed -f cardwords_sgml2helptext.sed \
| ./cardwords_sgml2helptext > $1.help_text
