/*
  This file is part of CardWords.
  (C) 1999 Tobias Peters

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_playerlist.hh

#ifndef CARDWORDS_GTK_PLAYERLIST_HH
#define CARDWORDS_GTK_PLAYERLIST_HH

#include "cardwords_playerlist.hh"
#include <vector>
#include <gtk--/clist.h>

class CardWords_Gtk_PlayerList : public Gtk::CList
{
public:
  CardWords_Gtk_PlayerList(const CardWords_PlayerList *);
  void
  update(const CardWords_PlayerList *);

private:
  size_t min_width;
};

#endif
