/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_hand.hh

#ifndef CARDWORDS_HAND_HH
#define CARDWORDS_HAND_HH

// The hand is what represents a player for cardwords.
// The hand holds the cards that a player can use.

#include "cardwords_cardcontainer1dim.hh"
#include "cardwords_pile.hh"

class CardWords_Hand : public CardWords_CardContainer1Dim {
public:
  // drawing a card from the pile
  CardWords_Card *
  draw_card (CardWords_Pile *);
  CardWords_Hand(size_t size) : CardWords_CardContainer1Dim(size){};
};

inline CardWords_Card * CardWords_Hand::draw_card (CardWords_Pile * pile)
{
  if (free_places() == 0) {
    return 0;
  }

  CardWords_Card * card = pile->produce_card();
  if (card != 0) {
    add_card(card);
  }
  return card;
}

#endif
