/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_searchwordmusts.hh

#ifndef CARDWORDS_SEARCHWORDMUSTS_HH
#define CARDWORDS_SEARCHWORDMUSTS_HH

#include "cardwords_char.hh"
#include "cardwords_string.hh"
#include <map>

typedef  pair<size_t, CardWords_Char> CardWords_SearchWordMust;

class CardWords_SearchWordMusts : public map<size_t, CardWords_Char>     
{
public:
  // this one increments the position of every single must:
  void increment (void);

  // this adds the must CardWords_SearchWordMust (0, must_kc):
  void add (const CardWords_Char & must_kc)
    {insert(value_type(0,must_kc));};

  // combination of a) incrementing and b) adding:
  void increment (const CardWords_Char & must_kc)
    {increment();add(must_kc);};

  // checks if the word follows the musts:
  bool check_word (const CardWords_String &) const;
  
};

#endif
