/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_dicbot_cardtablemustnot.hh

/* This file defines the class CardWords_DicBot_CardTableMustNot. The class
   contains characters that are forbidden at a card-table cell. Implementaion
   is a set. */
#ifndef CARDWORDS_DICBOT_CARDTABLEMUSTNOT_HH
#define CARDWORDS_DICBOT_CARDTABLEMUSTNOT_HH
#include <set>
#include "cardwords_char.hh"

class CardWords_DicBot_CardTableMustNot : public set<CardWords_Char> {
public:
  inline bool test_allowed (CardWords_Char c) const {
    return find(c) == end();
  }
  
  inline bool test_forbidden (CardWords_Char c) const {
    return find(c) != end();
  }

  inline void clear (void) { erase(begin(),end()); }
};
#endif
