/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_startserverdialog.hh

#ifndef CARDWORDS_GTK_STARTSERVERDIALOG_HH
#define CARDWORDS_GTK_STARTSERVERDIALOG_HH

#include "cardwords_gtk_helpwindow.hh"
#include <strstream.h>
#include <vector>
#include "cardwords_options.hh"
#include <gtk--/table.h>
namespace Gtk {class Entry;}
namespace Gtk {class Button;}
namespace Gtk {class FileSelection;}

class CardWords_Gtk_StartServerDialog : public Gtk::Table {
public:
  SigC::Signal1<void, const string &> help_signal; // arg is the help_text pointer

  // Put the entries' contents into the options object.
  void
  store_options(void);

  CardWords_Gtk_StartServerDialog(CardWords_Options *);
  ~CardWords_Gtk_StartServerDialog();
private:
  // [0] - server program
  // [1] - charset file
  // [2] - cardset file
  // [3] - cardtable file
  // [4] - bonus file
  // [5] - dictionary file
  // [6] - personal dictionary file
  // [7] - number of players
  // [8] - number of robots
  // [9] - number of watchers
  // [10]- port number

  void
  emit_help_signal(void);
  void
  show_file_selection(int);
  void
  file_selection_dialog_ok(int index);
  int
  file_selection_dialog_delete(GdkEventAny * /* , int index */);
  void
  file_selection_dialog_cancel(int index);

  Gtk::Label * label[11];
  Gtk::Entry * entry[11];
  Gtk::Adjustment * adjustment[11];
  Gtk::Button * choose_file_button[11];
  Gtk::FileSelection * file_selection[11];
  
  Gtk::Button * help_button;

  CardWords_Options * options;
};

#endif
