/* This file is part of cardwords
   (c) 1999 Tobias Peters
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_max.hh

#ifndef CARDWORDS_MAX_HH
#define CARDWORDS_MAX_HH
// the standard inline template max and min functions:
template<class type>
inline type
cardwords_max (type a, type b)
{
  return (a < b) ? b : a;
}

template<class type>
inline type
cardwords_min (type a, type b)
{
  return (a < b) ? a : b;
}
#endif
