/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_addword.hh

#ifndef CARDWORDS_GTK_ADDWORD_HH
#define CARDWORDS_GTK_ADDWORD_HH

#include <gtk--/box.h>
#include "cardwords_string.hh"
namespace Gtk {class Button;}

class CardWords_GtkClient;

class CardWords_Gtk_AddWord : public Gtk::VBox
{
public:
  CardWords_Gtk_AddWord(CardWords_GtkClient *);
  ~CardWords_Gtk_AddWord();

  void
  add_word(const CardWords_String &);

private:
  void
  yes_button_callback(void);
  void
  no_button_callback(void);
  void
  retry_button_callback(void);
  
  CardWords_GtkClient * gtkclient;
  Gtk::Label  * question;
  Gtk::HBox   * button_box;
  Gtk::Button * yes_button;
  Gtk::Button * no_button;
  Gtk::Button * retry_button;
  
  CardWords_String word;
};

#endif
