/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_implication.hh

/* This file defines the logical function bool implication(bool,bool).
   Its main purpose is to ease the read of some assert statements.
*/

#ifndef CARDWORDS_IMPLICATION_HH
#define CARDWORDS_INPLICATION_HH

inline bool implication(bool if_condition, bool then_condition)
{
  return (!if_condition) || then_condition;
}
#endif
