/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_pileinfo.cc

#include "cardwords_gtk_pileinfo.hh"
#include <strstream.h>
#include "cardwords_gettext.hh"
#include <gtk--/label.h>


// CardWords_Gtk_PileInfo is a simple class displaying how many cards are left
// in the pile.
CardWords_Gtk_PileInfo::CardWords_Gtk_PileInfo(size_t initial_no = 0)
{
  set_homogeneous(false);
  
  title_label = new Gtk::Label(gettext("Number of remaining cards in pile:"));
  count_label = new Gtk::Label("");

  update_no_of_cards(initial_no);// Sets the count_label.

  pack_start(*title_label, false, false);
  pack_start(*count_label, false, false);

  title_label->show();
  count_label->show();
}

CardWords_Gtk_PileInfo::~CardWords_Gtk_PileInfo()
{
  count_label->hide();
  remove(*count_label);
  delete count_label;

  title_label->hide();
  remove(*title_label);
  delete title_label;
}  

void
CardWords_Gtk_PileInfo::update_no_of_cards(size_t no)
{
  ostrstream count_stream;
  count_stream << no << ends;
  count_stream.freeze(true);
  count_label->set(count_stream.str());
  count_stream.freeze(false);

  no_of_cards = no;
}
