/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_optionsdialog.hh

#ifndef CARDWORDS_GTK_OPTIONSDIALOG_HH
#define CARDWORDS_GTK_OPTIONSDIALOG_HH

#include "cardwords_gtk_startserverdialog.hh"
#include "cardwords_gtk_connectdialog.hh"
#include "cardwords_gtk_appearancedialog.hh"
#include "cardwords_gtk_identitydialog.hh"
#include "cardwords_options.hh"
#include <gtk--/notebook.h>

class CardWords_Gtk_OptionsDialog : public Gtk::Notebook
{
public:
  CardWords_Gtk_OptionsDialog(CardWords_Options *);
  ~CardWords_Gtk_OptionsDialog();

  // Put the entries' contents into the options object.
  void
  store_options(void);

  SigC::Signal1<void, const string &> help_signal; // arg is the help_text pointer

private:
  CardWords_Gtk_IdentityDialog *    identity_dialog;
  CardWords_Gtk_AppearanceDialog *  appearance_dialog;
  CardWords_Gtk_StartServerDialog * start_server_dialog;
  CardWords_Gtk_ConnectDialog *     connect_dialog;

  Gtk::Label * identity_label;
  Gtk::Label * appearance_label;
  Gtk::Label * start_server_label;
  Gtk::Label * connect_label;
  
  CardWords_Options * options;
};
#endif
