/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_menu.hh

#ifndef CARDWORDS_GTK_MENU_HH
#define CARDWORDS_GTK_MENU_HH

#include <list>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
namespace Gtk {class MenuItem;}

class CardWords_Gtk_MenuWithItems : public Gtk::Menu
{
private:
  list<Gtk::MenuItem *> menu_items;
  
public:
    CardWords_Gtk_MenuWithItems(const list<string> & list_of_menu_item_names);
    ~CardWords_Gtk_MenuWithItems();

    Gtk::MenuItem *
    get_menu_item(const string & label) const;
};

class CardWords_Gtk_MenuBar : public Gtk::MenuBar
{
private:
  list<CardWords_Gtk_MenuWithItems *> menus;
  list<Gtk::MenuItem *> menu_names;

public:
  CardWords_Gtk_MenuBar(void);
  ~CardWords_Gtk_MenuBar();

  void
  add_menu(const string &       menu_name,
           const list<string> & list_of_menu_item_names);

  Gtk::MenuItem *
  get_menu_item(const string & menu_name, const string & menu_item_name) const;
};


#endif
