/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_stringconverters.hh

#include <string>

#ifndef CARDWORDS_STRINGCONVERTERS_HH
#define CARDWORDS_STRINGCONVERTERS_HH

typedef int CardWords_StringConverterFlags;

const CardWords_StringConverterFlags CSC_DEFAULT        = 0;
const CardWords_StringConverterFlags CSC_LEADING_ZEROES = 1 << 0;
const CardWords_StringConverterFlags CSC_STRICT_SIZE    = 1 << 1;
const CardWords_StringConverterFlags CSC_MINIMUM_SIZE   = 1 << 2;

unsigned
string2unsigned(const string &);

string
unsigned2string(unsigned,
		CardWords_StringConverterFlags = CSC_DEFAULT,
		size_t length = 0);

string
int2string(int,
           CardWords_StringConverterFlags = CSC_DEFAULT,
           size_t length = 0);
#endif
