/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_readfile.hh

#ifndef CARDWORDS_READFILE_HH
#define CARDWORDS_READFILE_HH

#include <string> // getting rid of "charpointer.hh"
#include <strstream.h>

// CardWords_ReadFile is an object that reads a whole file into memory.

class CardWords_ReadFile
{
public:
  CardWords_ReadFile(const string & filename);
  CardWords_ReadFile(const CardWords_ReadFile &);
  ~CardWords_ReadFile();

  // successful constructed or not?
  operator bool() const;
  
  // return the files contents:
  const string &
  str(void) const;
private:
  string      text_file_contents;
  bool        successful;
};

#endif

