/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_clientoptionsdialog.hh

#ifndef CARDWORDS_GTK_CLIENTOPTIONSDIALOG_HH
#define CARDWORDS_GTK_CLIENTOPTIONSDIALOG_HH

#include <gtk--/table.h>
#include <gtk--/adjustment.h>
namespace Gtk {class Button;}
namespace Gtk {class HBox;}
namespace Gtk {class CheckButton;}
namespace Gtk {class Entry;}
namespace Gtk {class SpinButton;}

class CardWords_Gtk_CientOptionsDialog : public Gtk::Table
{
public:
  SigC::Signal0<void> local_button_clicked;
  SigC::Signal0<void> remote_button_clicked;
  SigC::Signal1<void, const string &> help_signal; // arg is the help_text pointer
  
  string
  get_name(void) const;
  string
  get_alt_name(void) const;

  size_t
  get_cell_width() const;
  size_t
  get_cell_height() const;

  CardWords_Gtk_CientOptionsDialog();
  ~CardWords_Gtk_CientOptionsDialog();

  bool
  does_user_want_to_be_watcher(void) const;
private:
  void
  emit_help_signal(void);
  void
  player_button_toggled(void);
  void
  watcher_button_toggled(void);
  bool currently_handling_toggle;

  Gtk::Button * local_button;
  Gtk::Button * remote_button;
  Gtk::Button * help_button;
  Gtk::HBox   * button_box;
  

  Gtk::CheckButton * player_button;
  Gtk::CheckButton * watcher_button;
  Gtk::HBox        * radio_box;
  
  Gtk::Label * name_label;
  Gtk::Entry * name_entry;
  
  Gtk::Label * alt_name_label;
  Gtk::Entry * alt_name_entry;

  Gtk::Label * cell_width_label;
  Gtk::SpinButton * cell_width_entry;
  Gtk::Adjustment cell_width_adjustment;
  
  Gtk::Label * cell_height_label;
  Gtk::SpinButton * cell_height_entry;
  Gtk::Adjustment  cell_height_adjustment;
};

#endif
