/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_cardselection.cc

#include <iterator.h>
#include <algo.h>
#include "cardwords_cardselection.hh"
#include "cardwords_gettext.hh"
#include "cardwords_exceptions.hh"

istream & operator >>( istream & istream, CardWords_CardSelection & sel)
{
  size_t number = 772227722;
  istream >> number;
  if (istream.bad() || number == 772227722) {
    throw CardWords_XBadInput("CardWords_CardSelection",
			    gettext("number"));
  }
  ;
  sel.clear();

  /* copy_n does not terminate. don't know why. */
  /*
    istream_iterator<CardWords_CardDescription> isiter (istream);
    back_insert_iterator<CardWords_CardSelection> biiter (sel);
    copy_n(isiter, number, biiter);
  */

  // so I use a for-loop:
  CardWords_CardDescription desc;
  for (size_t index = 0; index < number; index++) {
    istream >> desc;
    sel.push_back(desc);
  }
  return istream;
}

ostream & operator <<( ostream & ostream, const CardWords_CardSelection &sel)
{
  ostream << sel.size() << ' ';
  copy (sel.begin(), sel.end(),
	ostream_iterator<CardWords_CardDescription>(ostream," "));
  return ostream;
}
