/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_direction.hh

#ifndef CARDWORDS_DIRECTION_HH
#define CARDWORDS_DIRECTION_HH
#include "cardwords_iostream.hh"
#include "cardwords_exceptions.hh"

enum CardWords_Direction {horizontal, vertikal};

inline
CardWords_Direction operator !(CardWords_Direction d)
{
  return (d == horizontal) ? vertikal : horizontal;
}

inline
ostream & operator << (ostream & ostream, CardWords_Direction direction)
{
  return ostream << ( (direction == horizontal) ?   '_'   :   '|'   );
}

inline
istream & operator >> (istream &istream, CardWords_Direction &direction)
{
  char ch = 0;
  while (istream && (ch == 0 || ch == ' ' || ch == '\n' || ch == '\t')) {
    istream >> ch;
  }

  switch (ch) {
  case '_':
    direction = horizontal;
    break;
  case '|':
    direction = vertikal;
    break;
  default:
    throw CardWords_XBadInput ("CardWords_Direction",
                               gettext("\'_\' or \'|\'"));
  }
  return istream;
}
#endif
