/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_carddescription.cc

#include "cardwords_carddescription.hh"
#include "cardwords_exceptions.hh"

CardWords_CardDescription::CardWords_CardDescription (class istream & istream)
  : CardWords_CardBase(' ')
{
  istream >> *this;
}

class istream &
operator >> (class istream &istream, CardWords_CardDescription &desc)
{
  char ch = 0;
  assert (bool(istream));
  if (!istream) {
    throw CardWords_XBadInput("CardWords_CardDescription","\'[\'");
  }

  // read opening bracket:
  while (istream && ch != '[') {
    istream >> ch;
    // allow only whitespace between cards
    assert(ch == '[' || ch == ' ' || ch == '\n' || ch == '\t');
    if (ch != '[' && ch != ' '&& ch != '\n' && ch != '\t') {
      throw CardWords_XBadInput("CardWords_CardDescription","\'[\'");
    }
  }

  // Read the face value and assign it to the meaning. This means, we
  // do not allow to specify a meaning through the constructor.
  // The reason is, it is impossible to store the face value here,
  // this has to be done in the derived class which does not exist right
  // now (this base class is just being constructed!)
  assert (bool(istream));
  if (!istream) {
    throw CardWords_XBadInput("CardWords_CardDescription",
                              gettext("face value"));
  }
  CardWords_Char kc;
  istream >> kc;
  desc.set_faceValue(kc);

  // read the meaning in the istream:
  assert (bool(istream));
  if (!istream) {
    throw CardWords_XBadInput("CardWords_CardDescription",gettext("meaning"));
  }
  istream >> kc;
  desc.set_meaning(kc);

  // now read the points:
  assert (bool(istream));
  if (!istream) {
    throw CardWords_XBadInput("CardWords_CardDescription",gettext("score"));
  }
  int points;
  istream >> points;
  desc.set_points(points);

  // read the closing bracket:
  istream >> ch;
    
  if (ch != ']') {
    throw CardWords_XBadInput("CardWords_CardDescription",
                            gettext("meaning"));
  }
  return istream;
}


