/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_cardoccurrencemap.cc

#include "cardwords_cardoccurrencemap.hh"
#include <strstream.h>

CardWords_CardOccurrenceMap::
CardWords_CardOccurrenceMap(const string & cardsel)
{
  istrstream istr(cardsel.c_str());

  size_t number_of_cards;

  istr >> number_of_cards;

  for (size_t index = 0; index < number_of_cards; index++) {
    assert(istr.good());
    CardWords_CardDescription desc(istr);
    if (find(desc) != end()) {
      ++(*this)[desc];
    }
    else {
      (*this)[desc] = 1;
    }
  }
}
