/*
  This file is part of CardWords.
  (C) 1999 Tobias Peters

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// config.hh

#ifndef CONFIG_HH
#define CONFIG_HH

#define AUTOSTART_WHEN_ENOUGH_LOGGED_ON
//#define ALLOW_ROBOTS_ONLY_GAME

#define DEFAULT_WORD_FACTOR   1
#define DEFAULT_LETTER_FACTOR 1
#define DEFAULT_CELL_POINTS   0
#define DEFAULT_CELL_BLOCKED  false

#endif
