/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_cardtablelocation.hh

#ifndef CARDWORDS_CARDTABLELOCATION_HH
#define CARDWORDS_CARDTABLELOCATION_HH

#include <pair.h>
#include "cardwords_iostream.hh"
#include "cardwords_exceptions.hh"

typedef pair<size_t,size_t> CardWords_CardTableLocation;

inline
ostream & operator << (ostream & ostream,
                       const CardWords_CardTableLocation & loc)
{
  return ostream << '(' << loc.first << ',' << loc.second << ')';
}

inline
istream & operator >> (istream & istream, CardWords_CardTableLocation & loc)
{
  char ch = ' ';
  while (istream && (ch == ' ' || ch == '\n' || ch == '\t')) {
    istream >> ch;
  }
  if (ch != '(') {
    throw CardWords_XBadInput ("CardWords_CardTableLocation", "\'(\'");
  }
  istream >> loc.first;
  if (istream.bad())
    throw CardWords_XBadInput ("CardWords_CardTableLocation",
                               gettext("number"));

  ch = ' ';
  while (istream && (ch == ' ' || ch == '\n' || ch == '\t')) {
    istream >> ch;
  }
  if (ch != ',') {
    throw CardWords_XBadInput ("CardWords_CardTableLocation",  "\',\'"  );
  }
  
  istream >> loc.second;
  if (istream.bad())
    throw CardWords_XBadInput ("CardWords_CardTableLocation",
                               gettext("number"));

  ch = ' ';
  while (istream && (ch == ' ' || ch == '\n' || ch == '\t')) {
    istream >> ch;
  }
  if (ch != ')') {
    throw CardWords_XBadInput ("CardWords_CardTableLocation",  "\')\'"  );
  }
  return istream;
}
#endif
