/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_matrix.hh

// CardWords_Matrix translates a vector into a 2-dimensional matrix

#ifndef CARDWORDS_MATRIX_HH
#define CARDWORDS_MATRIX_HH

#include <vector>
#include "cardwords_cardtablelocation.hh"

template <class T>
class CardWords_Matrix {
public:
  typedef vector<T>::reference              reference;
  typedef vector<T>::const_reference        const_reference;

  size_t
  size() const;

  size_t
  get_width(void) const;

  size_t
  get_height(void) const;

  reference
  operator()(size_t column, size_t row);
  const_reference
  operator()(size_t column, size_t row) const;

  reference
  operator[](const CardWords_CardTableLocation &);
  const_reference
  operator[](const CardWords_CardTableLocation &) const;

  CardWords_Matrix(size_t set_width, size_t set_height, const T& value);

private:
  vector<T> representation;
  size_t    width, height;
};


template <class T>
inline
size_t
CardWords_Matrix<T>::size() const
{
  return representation.size();
}

template <class T>
inline
size_t
CardWords_Matrix<T>::get_width(void) const
{
  return width;
}

template <class T>
inline
size_t
CardWords_Matrix<T>::get_height(void) const
{
  return height;
}

template <class T>
inline
CardWords_Matrix<T>::reference
CardWords_Matrix<T>::operator() (size_t column, size_t row)
{
  return representation[column + row * width];
}

template <class T>
inline
CardWords_Matrix<T>::const_reference
CardWords_Matrix<T>::operator() (size_t column, size_t row) const
{
  return representation[column + row * width];
}

template <class T>
inline
CardWords_Matrix<T>::reference
CardWords_Matrix<T>::operator[](const CardWords_CardTableLocation & loc)
{
  return representation[loc.first + loc.second * width];
}

template <class T>
inline
CardWords_Matrix<T>::const_reference
CardWords_Matrix<T>::operator[](const CardWords_CardTableLocation & loc) const
{
  return representation[loc.first + loc.second * width];
}

template <class T>
inline
CardWords_Matrix<T>::CardWords_Matrix(size_t set_width,
                                  size_t set_height,
                                  const T& value = T())
  : representation (set_width * set_height, value),
    width (set_width),
    height(set_height)
{}
  
#endif
