/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_helpwindow.hh

#ifndef CARDWORDS_GTK_HELPWINDOW_HH
#define CARDWORDS_GTK_HELPWINDOW_HH

#include <gtk--/window.h>
namespace Gtk {class Text;}

// CardWords_Gtk_HelpWindow is a window displaying a help text on demand.


class CardWords_Gtk_HelpWindow : public Gtk::Window
{
public:  
  CardWords_Gtk_HelpWindow();
  ~CardWords_Gtk_HelpWindow();

  void
  show_help_text(const string & help_text);

protected:
  virtual gint
  delete_event_impl(GdkEventAny* p1);
  
private:
  int
  CardWords_Gtk_HelpWindow::hide_help_window(GdkEventAny *);
  
  Gtk::Text * text;
};

#endif
