/* This file is part of cardwords
   (c) 1998 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_card.cc

#include "cardwords_card.hh"
#include "cardwords_iostream.hh"
#include "cardwords_exceptions.hh"

long unsigned int CardWords_Card::nextUniqueNumber = 0;

CardWords_Card::CardWords_Card( istream & istream )
  : CardWords_CardBase (istream),
    container (0),
    uniqueNumber (nextUniqueNumber++)
{
  // now read the points:
  assert (bool(istream));
  if (!istream) {
    throw CardWords_XBadInput("CardWords_CardDescription","score");
  }
  istream >> points;

  char ch;
  istream >> ch;
    
  if (ch != ']') {
    throw CardWords_XBadInput("CardWords_CardDescription","\']\'");
  }

  faceValue = get_meaning();
  wildcard = (faceValue == CardWords_Char::blanko);
}
