/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_cardtableinfo.hh

#ifndef CARDWORDS_GTK_CARDTABLEINFO_HH
#define CARDWORDS_GTK_CARDTABLEINFO_HH

#include "cardwords_cardtable.hh"
#include "cardwords_gtk_cardcell.hh"
#include <gtk--/clist.h>
namespace Gtk {class Label;}

class CardWords_Gtk_CardTableInfo : public Gtk::VBox {
public:
  void show_info_about_cell(const CardWords_Gtk_CardCell *);

  CardWords_Gtk_CardTableInfo(const CardWords_CardTable *, // The parameter is
                              //                           for the dimensions.
                              bool use_digits_for_the_columns,
                              // and chars for the rows, otherwise the other
                              // way round

                              bool column_indices_grow_rightwards,
                              // if true, the rightmost column has the
                              // greatest index

                              bool row_indices_grow_upwards);

                        
  ~CardWords_Gtk_CardTableInfo();
  
private:
  const size_t card_table_width, card_table_height;

  size_t column_width;
  
  Gtk::CList * list;

  // strings in the rows of the Gtk::CList:
  vector<vector<string> > info_rows; //[9][2];
  
  Gtk::Label * title_label;
  
  bool use_digits_for_the_columns;
  // and chars for the rows, otherwise the other way
  // round
                          
  bool column_indices_grow_rightwards;
  // if true, the rightmost column has the greatest
  // index
                          
  bool row_indices_grow_upwards;
};
#endif
