/* This file is part of cardwords
   (c) 1999 Tobias Peters
   see file COPYING for the copyright terms.
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// cardwords_gtk_identitydialog.hh

#ifndef CARDWORDS_GTK_IDENTITYDIALOG_HH
#define CARDWORDS_GTK_IDENTITYDIALOG_HH

#include "cardwords_options.hh"
#include <gtk--/table.h>
namespace Gtk {class Button;}
namespace Gtk {class Entry;}

class CardWords_Gtk_IdentityDialog : public Gtk::Table
{
public:
  CardWords_Gtk_IdentityDialog(CardWords_Options *);
  ~CardWords_Gtk_IdentityDialog();
  
  SigC::Signal1<void, const string &> help_signal; // arg is the help_text

  // Put the entries' contents into the options object.
  void
  store_options(void);

private:
  void
  emit_help_signal(void);

  Gtk::Button * help_button;
  
  Gtk::Label * name_label;
  Gtk::Entry * name_entry;
  
  Gtk::Label * alt_name_label;
  Gtk::Entry * alt_name_entry;

  CardWords_Options * options;
};

#endif
